
#include <string.h>
#include "sparcommon.h"
#include "wiced_bt_dev.h"
#include "wiced_bt_uuid.h"
#include "wiced_bt_ble.h"
#include "wiced_bt_gatt.h"
#include "wiced_bt_cfg.h"
#include "wiced_bt_trace.h"
#include "wiced_hal_nvram.h"
#include "wiced_transport.h"

#if ( defined(CYW20706A2) || defined(CYW20719B1) || defined(CYW20719B0) || defined(CYW20721B1) || defined(CYW20735B0) || defined(CYW43012C0) )
#include "wiced_bt_app_common.h"
#endif
#include "wiced_platform.h"
#include "wiced_bt_stack.h"
#include "wiced_memory.h"
#include "wiced_hal_puart.h"
#include "wiced_timer.h"

extern const wiced_bt_cfg_settings_t wiced_bt_cfg_settings;
extern const wiced_bt_cfg_buf_pool_t wiced_bt_cfg_buf_pools[];

/***************************************************************************************/

const wiced_transport_cfg_t  transport_cfg =
{
    WICED_TRANSPORT_UART,
    { WICED_TRANSPORT_UART_HCI_MODE, HCI_UART_DEFAULT_BAUD },
    { 0, 0},
    NULL,
    NULL,
    NULL
};




/***************************************************************************************/

uint8_t DeviceNo = 0;


static wiced_result_t           Observer_management_cback( wiced_bt_management_evt_t event,  wiced_bt_management_evt_data_t *p_event_data );
static void                     Observer_app_init( void );
static void                     Observer_scan_result_cback( wiced_bt_ble_scan_results_t *p_scan_result, uint8_t *p_adv_data );

/* Observer application start , ie, entry point to the application.
 * It is mandatory for all the applications to define this
 */
APPLICATION_START()
{
    wiced_transport_init( &transport_cfg );
#ifdef WICED_BT_TRACE_ENABLE
    // Set the debug uart as WICED_ROUTE_DEBUG_NONE to get rid of prints
    // wiced_set_debug_uart(WICED_ROUTE_DEBUG_NONE);

    // Set to PUART to see traces on peripheral uart(puart)
    wiced_set_debug_uart( WICED_ROUTE_DEBUG_TO_PUART );
    wiced_hal_puart_select_uart_pads( WICED_PUART_RXD, WICED_PUART_TXD, 0, 0);

    // Set to HCI to see traces on HCI uart - default if no call to wiced_set_debug_uart()
    // wiced_set_debug_uart( WICED_ROUTE_DEBUG_TO_HCI_UART );

    // Use WICED_ROUTE_DEBUG_TO_WICED_UART to send formatted debug strings over the WICED
    // HCI debug interface to be parsed by ClientControl/BtSpy.
    // Note: WICED HCI must be configured to use this - see wiced_trasnport_init(), must
    // be called with wiced_transport_cfg_t.wiced_tranport_data_handler_t callback present
    // wiced_set_debug_uart(WICED_ROUTE_DEBUG_TO_WICED_UART);
#endif

    // WICED BT Stack initialization and registering the managment callback
    // init complete
    wiced_bt_stack_init( Observer_management_cback,
                    &wiced_bt_cfg_settings, wiced_bt_cfg_buf_pools );
}


/*
 * Observer bt/ble device and link management callbacks
 */
wiced_result_t Observer_management_cback( wiced_bt_management_evt_t event,  wiced_bt_management_evt_data_t *p_event_data)
{
    wiced_result_t               result = WICED_BT_SUCCESS;
    wiced_result_t               add_db_result;

    WICED_BT_TRACE("Observer_management_cback: %x\r\r\n", event );

    switch( event )
    {
        /* Bluetooth  stack enabled */
        case BTM_ENABLED_EVT:
            Observer_app_init( );
            break;

        case BTM_BLE_SCAN_STATE_CHANGED_EVT:
            WICED_BT_TRACE( "Scan State Change: %d\r\n", p_event_data->ble_scan_state_changed );
            break;

        default:
            break;
    }

    return result;
}

void Observer_app_init( void )
{
    int                    index;
    wiced_bt_gatt_status_t gatt_status;

    wiced_bool_t start = 1;
    uint8_t duration = 0;

    WICED_BT_TRACE( "Observer_app_init\r\n" );


#ifdef ENABLE_HCI_TRACE
    /* Register callback for receiving hci traces */
    wiced_bt_dev_register_hci_trace( hello_sensor_hci_trace_cback );
#endif

    wiced_bt_ble_observe (start,duration , Observer_scan_result_cback);

}

/*
 * This function handles the scan results
 */
void Observer_scan_result_cback( wiced_bt_ble_scan_results_t *p_scan_result, uint8_t *p_adv_data )
{
    wiced_result_t         status;
    wiced_bool_t           ret_status;
    uint8_t                length;
    uint8_t                i = 0;
    uint8_t *              p_data;

    if ( p_scan_result )
    {

        WICED_BT_TRACE("\r\n");

        WICED_BT_TRACE( "Adv packet received\r\n");
        WICED_BT_TRACE( " Peer device Address : ( %B ) \r\n",p_scan_result->remote_bd_addr);
        WICED_BT_TRACE( " RSSI : %d  \r\n",p_scan_result->rssi );
        WICED_BT_TRACE( " Advertising data of peer: " );
        for(i=0;i<31;i++)
        {
            WICED_BT_TRACE( "%d ",p_adv_data[i]);

        }
        WICED_BT_TRACE("\r\n");

    }
    else
    {
        WICED_BT_TRACE( "Scan completed :\r\n" );
    }
}
